from __future__ import print_function

from datetime import datetime, timedelta, tzinfo

from ..ago import human
from ..ago import delta2dict

# datetime objects
PRESENT = datetime.now()
PAST = PRESENT - timedelta(492, 58711, 45)  # days, secs, microseconds
FUTURE = PRESENT + timedelta(2, 12447, 967742)  # days, secs, microseconds

# timedelta objects
PAST_DELTA = PRESENT - PAST
FUTURE_DELTA = PRESENT - FUTURE
ONE_YEAR_FOUR_HOURS_DELTA = timedelta(365, 14400, 0)


class FixedOffset(tzinfo):
    """Fixed offset in minutes east from UTC.

    Adapted from Python's documentation about tzinfo objects:
    https://docs.python.org/2.7/library/datetime.html#tzinfo-objects
    """

    def __init__(self, offset):
        self._offset = timedelta(minutes=offset)

    def utcoffset(self, dt):
        return self._offset

    def tzname(self, dt):
        return "Test"

    def dst(self, dt):
        return timedelta(0)


def test_human_passed_datetime_is_string():
    assert isinstance(human(PAST), str)


def test_human_passed_timedelta_is_string():
    assert isinstance(human(PAST_DELTA), str)


def test_delta2dict_is_dict():
    assert isinstance(delta2dict(PAST_DELTA), dict)


def test_ago_in_past_human():
    assert "ago" in human(PAST)


def test_in_in_future_human():
    assert "in" in human(FUTURE)


def test_no_coma_in_one_precision():
    assert "," not in human(PAST, precision=1)
    assert "," not in human(FUTURE, precision=1)


def test_coma_in_three_precision():
    assert "," in human(PAST, precision=3)
    assert "," in human(FUTURE, precision=3)


def test_coma_in_out_of_bounds_precision():
    assert "," in human(PAST, precision=10)
    assert "," in human(FUTURE, precision=10)


def test_zero_day_is_skipped_display_hour():
    _result = human(ONE_YEAR_FOUR_HOURS_DELTA, precision=2)
    assert "year" in _result
    # day is 0 so it is skipped, so we should show hours ...
    assert "hour" in _result


def test_one_day_singular():
    assert "s" not in human(timedelta(1))


def test_two_day_plural():
    assert "s" in human(timedelta(2))


def test_abbreviation():
    assert "2d ago" in human(timedelta(2), abbreviate=True)
    assert "3d, 12h ago" in human(timedelta(3.5), abbreviate=True)
    assert "2h, 24m ago" in human(timedelta(.1), abbreviate=True)
    assert "1y, 35d ago" in human(timedelta(400), abbreviate=True)


def test_timestamp_integer():
    result = human(1474485467933 / 1000, precision=6)
    assert "minute" in result
    assert "second" in result
    assert "ago" in result


def test_timestamp_float():
    result = human(1474485467933 / 1000.0, precision=6)
    assert "minute" in result
    assert "second" in result
    assert "ago" in result


def test_past_tense():
    output = human(
        PAST,
        past_tense="titanic sunk {} ago",
        future_tense="titanic will sink in {} from now",
    )
    assert "titanic sunk" in output


def test_future_tense():
    output = human(
        FUTURE,
        past_tense="titanic sunk {} ago",
        future_tense="titanic will sink in {} from now",
    )
    assert "titanic will sink in" in output


def test_valid_past_dict():
    past_dict = delta2dict(PAST_DELTA)
    assert past_dict["year"] == 1
    assert past_dict["day"] == 127
    assert past_dict["hour"] == 16
    assert past_dict["minute"] == 18
    assert past_dict["microsecond"] == 45


def test_valid_future_dict():
    past_dict = delta2dict(FUTURE_DELTA)
    assert past_dict["year"] == 0
    assert past_dict["day"] == 2
    assert past_dict["hour"] == 3
    assert past_dict["minute"] == 27
    assert past_dict["millisecond"] == 967
    assert past_dict["microsecond"] == 742


def test_timezone_support():
    # test a naive datetime with no timezone
    # (add an extra minute to ensure it stays over an hour ahead during test)
    dt = datetime.now() + timedelta(minutes=61)
    output = human(dt)
    assert output.startswith("in 1 hour")

    # test a timezone-aware datetime with a UTC-2 timezone
    dt = datetime.now(tz=FixedOffset(-120)) + timedelta(minutes=61)
    output = human(dt)
    assert output.startswith("in 1 hour")


def example_usage():
    """Test and example usage"""

    print("\nTest past tense:\n")
    print(delta2dict(PAST_DELTA))
    print("Commented " + human(PAST_DELTA, 1))
    print(human(PAST, past_tense="Commented {} ago"))

    print(human(ONE_YEAR_FOUR_HOURS_DELTA, past_tense="Posted {} ago"))

    print("\nTest future tense:\n")
    print(delta2dict(FUTURE_DELTA))
    print("Shutdown " + human(FUTURE_DELTA, 5))
    print(human(FUTURE, future_tense="Shutdown in {} from now"))
    print("")


if __name__ == "__main__":
    example_usage()



from datetime import datetime, timedelta

units = ("year", "day", "hour", "minute", "second", "millisecond", "microsecond")


def get_delta_from_subject(subject):
    """Convert the subject to a timedelta and return it."""
    if isinstance(subject, timedelta):
        return subject

    if isinstance(subject, datetime):
        dt = subject
    else:
        # if it's not a datetime, assume it's a unix timestamp
        try:
            subject = float(subject)
        except ValueError:
            # some unknown type that couldn't be converted to a float
            raise TypeError("Unsupported subject type")

        dt = datetime.fromtimestamp(subject)

    # return a timedelta from the datetime, using its timezone (if it has one)
    return datetime.now(tz=dt.tzinfo) - dt


def delta2dict(delta):
    """Accepts a delta, returns a dictionary of units"""
    delta = abs(delta)
    return {
        "year": int(delta.days / 365),
        "day": int(delta.days % 365),
        "hour": int(delta.seconds / 3600),
        "minute": int(delta.seconds / 60) % 60,
        "second": int(delta.seconds % 60),
        "millisecond": int(delta.microseconds / 1000),
        "microsecond": int(delta.microseconds % 1000),
    }


def human(
    subject, precision=2, past_tense="{} ago", future_tense="in {}", abbreviate=False
):
    """
    Accept a subject, return a human readable timedelta string.

    :param subject: a datetime, timedelta, or timestamp (integer / float) object
    :param precision: the desired amount of unit precision (default: 2)
    :param past_tense: the format string used for a past timedelta (default: '{} ago')
    :param future_tense: the format string used for a future timedelta (default: 'in {}')
    :param abbreviate: boolean to abbreviate units (default: False)

    :returns: Human readable timedelta string (Str)
    """
    delta = get_delta_from_subject(subject)
    the_tense = future_tense if delta < timedelta(0) else past_tense

    d = delta2dict(delta)

    hlist = []
    count = 0

    # start building up the output in the hlist.
    for unit in units:
        if count >= precision:
            break  # met precision
        if d[unit] == 0:
            continue  # skip 0's
        if abbreviate:
            if unit == "millisecond":
                abr = "ms"
            elif unit == "microsecond":
                abr = "um"
            else:
                abr = unit[0]
            hlist.append("{}{}".format(d[unit], abr))
        else:
            s = "" if d[unit] == 1 else "s"  # handle plurals
            hlist.append("{} {}{}".format(d[unit], unit, s))
        count += 1

    return the_tense.format(", ".join(hlist))



# installation: pip install ago

from setuptools import setup, find_packages

setup(
    name="ago",
    version="0.0.94",
    description="ago: Human readable timedeltas",
    keywords="ago human readable time deltas timedelta datetime timestamp",
    long_description=open("README.rst").read(),
    author="Russell Ballestrini",
    author_email="russell.ballestrini@gmail.com",
    url="https://git.unturf.com/python/ago",
    packages=find_packages(exclude="tests"),
    platforms=["All"],
    license="Public Domain",
    py_modules=["ago"],
    include_package_data=True,
)

# setup keyword args: http://peak.telecommunity.com/DevCenter/setuptools

# built and uploaded to pypi with this:
# python setup.py sdist bdist_egg register upload
